#include <stddef.h>
#include "opcode/loongarch.h"

struct loongarch_ASEs_option LARCH_opts =
{
  .ase_test = 0,
  .ase_fix = 0,
  .ase_float = 0,
  .ase_128vec = 0,
  .ase_256vec = 0,

  .addrwidth_is_32 = 0,
  .addrwidth_is_64 = 0,
  .rlen_is_32 = 0,
  .rlen_is_64 = 0,
  .la_local_with_abs = 0,
  .la_global_with_pcrel = 0,
  .la_global_with_abs = 0,

  .abi_is_lp32 = 0,
  .abi_is_lp64 = 0,
};

/* 预留。按理来说应该传入足够的信息使得这个函数可以返回指令长度 */
size_t loongarch_insn_length (insn_t insn)
{
  return insn ? 4 : 4; /* eliminate warning */
}

const char * const loongarch_r_normal_name[32] =
{
  "$r0", "$r1", "$r2", "$r3", "$r4", "$r5", "$r6", "$r7",
  "$r8", "$r9", "$r10", "$r11", "$r12", "$r13", "$r14", "$r15",
  "$r16", "$r17", "$r18", "$r19", "$r20", "$r21", "$r22", "$r23",
  "$r24", "$r25", "$r26", "$r27", "$r28", "$r29", "$r30", "$r31",
};

const char * const loongarch_r_lp64_name[32] =
{
  "$zero", "$ra", "$tp", "$sp", "$a0", "$a1", "$a2", "$a3",
  "$a4", "$a5", "$a6", "$a7", "$t0", "$t1", "$t2", "$t3",
  "$t4", "$t5", "$t6", "$t7", "$t8", "$x", "$fp", "$s0",
  "$s1", "$s2", "$s3", "$s4", "$s5", "$s6", "$s7", "$s8",
};

const char * const loongarch_r_lp64_name1[32] =
{
  "", "", "", "", "$v0", "$v1", "", "",
  "", "", "", "", "", "", "", "",
  "", "", "", "", "", "", "", "",
  "", "", "", "", "", "", "", "",
};

const char * const loongarch_f_normal_name[32] =
{
  "$f0", "$f1", "$f2", "$f3", "$f4", "$f5", "$f6", "$f7",
  "$f8", "$f9", "$f10", "$f11", "$f12", "$f13", "$f14", "$f15",
  "$f16", "$f17", "$f18", "$f19", "$f20", "$f21", "$f22", "$f23",
  "$f24", "$f25", "$f26", "$f27", "$f28", "$f29", "$f30", "$f31",
};

const char * const loongarch_f_lp64_name[32] =
{
  "$fa0", "$fa1", "$fa2", "$fa3", "$fa4", "$fa5", "$fa6", "$fa7",
  "$ft0", "$ft1", "$ft2", "$ft3", "$ft4", "$ft5", "$ft6", "$ft7",
  "$ft8", "$ft9", "$ft10", "$ft11", "$ft12", "$ft13", "$ft14", "$ft15",
  "$fs0", "$fs1", "$fs2", "$fs3", "$fs4", "$fs5", "$fs6", "$fs7",
};

const char * const loongarch_f_lp64_name1[32] =
{
  "$fv0", "$fv1", "", "", "", "", "", "",
  "", "", "", "", "", "", "", "",
  "", "", "", "", "", "", "", "",
  "", "", "", "", "", "", "", "",
};

const char * const loongarch_c_normal_name[8] =
{
  "$fcc0", "$fcc1", "$fcc2", "$fcc3", "$fcc4", "$fcc5", "$fcc6", "$fcc7",
};


const char * const loongarch_cr_normal_name[4] =
{
  "$scr0", "$scr1", "$scr2", "$scr3",
};

const char * const loongarch_v_normal_name[32] =
{
  "$vr0", "$vr1", "$vr2", "$vr3", "$vr4", "$vr5", "$vr6", "$vr7",
  "$vr8", "$vr9", "$vr10", "$vr11", "$vr12", "$vr13", "$vr14", "$vr15",
  "$vr16", "$vr17", "$vr18", "$vr19", "$vr20", "$vr21", "$vr22", "$vr23",
  "$vr24", "$vr25", "$vr26", "$vr27", "$vr28", "$vr29", "$vr30", "$vr31",
};

const char * const loongarch_x_normal_name[32] =
{
  "$xr0", "$xr1", "$xr2", "$xr3", "$xr4", "$xr5", "$xr6", "$xr7",
  "$xr8", "$xr9", "$xr10", "$xr11", "$xr12", "$xr13", "$xr14", "$xr15",
  "$xr16", "$xr17", "$xr18", "$xr19", "$xr20", "$xr21", "$xr22", "$xr23",
  "$xr24", "$xr25", "$xr26", "$xr27", "$xr28", "$xr29", "$xr30", "$xr31",
};

#define SOME_INFORMATION 0
const int ALWAYS_EXCLUSION = 1;
static int FIX_FOR_SOME_ARCH = 1;

static struct loongarch_opcode loongarch_test_opcodes[] = {
/* match,    mask,       name, format, macro, include, exclude, pinfo */
{
0, 0, "a insn always excluded",	"r0:5,r5:5,r10:5",
"insn_1 args_1"
"insn_2 args_2"
, 0, &ALWAYS_EXCLUSION, SOME_INFORMATION
},
{
0, 0,
"never disassemble when mask is 0"
"and we only expand marco when mask is 0 for assemble",
"r", "macro expand %1"
, 0, 0, 0
},
{0, 0, "wrong_insn", "r",
"throw_error use_fake_insn.for_example.something_goes_wrong_with_%1"
, 0, 0, 0},

{0, 0, "normal_insn", "r",
"overloading of insn for your arch", &FIX_FOR_SOME_ARCH, 0, 0},
{0, 0, "normal_insn", "l",
"throw_error if_you_dont_want", &FIX_FOR_SOME_ARCH, 0, 0},

{0, 0, "normal_insn", "r", "", 0, 0, 0},
{0, 0, "normal_insn", "l", "", 0, 0, 0},
{0} /* Terminate the list.  */
};

static struct loongarch_opcode loongarch_macro_opcodes[] = {
/* match,    mask,       name, format, macro, include, exclude, pinfo */
{0, 0, "throw_error", "l", "%f", 0, 0, 0},
{0, 0, "li", "r,sc", "%f", 0, 0, 0},
{0, 0, "dli", "r,sc", "%f", 0, 0, 0},

{0, 0, "la", "r,la", "la.global %1,%2", 0, 0, 0},

{0, 0, "la.global", "r,la", "la.pcrel %1,%2", &LARCH_opts.la_global_with_pcrel, 0, 0},
{0, 0, "la.global", "r,r,la", "la.pcrel %1,%2,%3", &LARCH_opts.la_global_with_pcrel, 0, 0},
{0, 0, "la.global", "r,la", "la.abs %1,%2", &LARCH_opts.la_global_with_abs, 0, 0},
{0, 0, "la.global", "r,r,la", "la.abs %1,%3", &LARCH_opts.la_global_with_abs, 0, 0},
{0, 0, "la.global", "r,l", "la.got %1,%2", 0, 0, 0},
{0, 0, "la.global", "r,r,l", "la.got %1,%2,%3", 0, 0, 0},

{0, 0, "la.local", "r,la", "la.abs %1,%2", &LARCH_opts.la_local_with_abs, 0, 0},
{0, 0, "la.local", "r,r,la", "la.abs %1,%3", &LARCH_opts.la_local_with_abs, 0, 0},
{0, 0, "la.local", "r,la", "la.pcrel %1,%2", 0, 0, 0},
{0, 0, "la.local", "r,r,la", "la.pcrel %1,%2,%3", 0, 0, 0},

{0, 0, "la.abs", "r,la",
"lu12i.w %1,%%abs(%2)>>12;"
"ori %1,%1,%%abs(%2)&0xfff;"
, &LARCH_opts.addrwidth_is_32, 0, 0},
{0, 0, "la.abs", "r,la",
"lu12i.w %1,%%abs(%2)<<32>>44;"
"ori %1,%1,%%abs(%2)&0xfff;"
"lu32i.d %1,%%abs(%2)<<12>>44;"
"lu52i.d %1,%1,%%abs(%2)>>52;"
, &LARCH_opts.addrwidth_is_64, 0, 0},

{0, 0, "la.pcrel", "r,la",
"pcaddu12i %1,%%pcrel(%2+0x800)<<32>>44;"
"addi.w %1,%1,%%pcrel(%2+4)-(%%pcrel(%2+4+0x800)>>12<<12);"
, &LARCH_opts.addrwidth_is_32, 0, 0},

{0, 0, "la.pcrel", "r,la",
"pcaddu12i %1,%%pcrel(%2+0x800)>>12;"
"addi.d %1,%1,%%pcrel(%2+4)-(%%pcrel(%2+4+0x800)>>12<<12);"
, &LARCH_opts.addrwidth_is_64, 0, 0},
{0, 0, "la.pcrel", "r,r,la",
"pcaddu12i %1,(%%pcrel(%3)-(%%pcrel(%3+0x80000000)>>32<<32))<<32>>44;"
"ori %2,$r0,(%%pcrel(%3+4)-(%%pcrel(%3+4+0x80000000)>>32<<32))&0xfff;"
"lu32i.d %2,%%pcrel(%3+8+0x80000000)<<12>>44;"
"lu52i.d %2,%2,%%pcrel(%3+12+0x80000000)>>52;"
"add.d %1,%1,%2;"
, &LARCH_opts.addrwidth_is_64, 0, 0},

{0, 0, "la.got", "r,l",
"pcaddu12i %1,(%%pcrel(_GLOBAL_OFFSET_TABLE_+0x800)+%%gprel(%2))<<32>>44;"
"ld.w %1,%1,%%pcrel(_GLOBAL_OFFSET_TABLE_+4)+%%gprel(%2)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+4+0x800)+%%gprel(%2))>>12<<12);"
, &LARCH_opts.addrwidth_is_32, 0, 0},

{0, 0, "la.got", "r,l",
"pcaddu12i %1,(%%pcrel(_GLOBAL_OFFSET_TABLE_+0x800)+%%gprel(%2))>>12;"
"ld.d %1,%1,%%pcrel(_GLOBAL_OFFSET_TABLE_+4)+%%gprel(%2)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+4+0x800)+%%gprel(%2))>>12<<12);"
, &LARCH_opts.addrwidth_is_64, 0, 0},
{0, 0, "la.got", "r,r,l",
"pcaddu12i %1,(%%pcrel(_GLOBAL_OFFSET_TABLE_)+%%gprel(%3)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+0x80000000)+%%gprel(%3))>>32<<32))<<32>>44;"
"ori %2,$r0,(%%pcrel(_GLOBAL_OFFSET_TABLE_+4)+%%gprel(%3)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+4+0x80000000)+%%gprel(%3))>>32<<32))&0xfff;"
"lu32i.d %2,(%%pcrel(_GLOBAL_OFFSET_TABLE_+8+0x80000000)+%%gprel(%3))<<12>>44;"
"lu52i.d %2,%2,(%%pcrel(_GLOBAL_OFFSET_TABLE_+12+0x80000000)+%%gprel(%3))>>52;"
"ldx.d %1,%1,%2;"
, &LARCH_opts.addrwidth_is_64, 0, 0},

{0, 0, "la.tls.le", "r,la",
"lu12i.w %1,%%tprel(%2)>>12;"
"ori %1,%1,%%tprel(%2)&0xfff"
, &LARCH_opts.addrwidth_is_32, 0, 0},
//{0, 0, "la.tls.le", "r,la",
//"lu12i.w %1,%%tprel(%2)>>12;"
//"ori %1,%1,%%tprel(%2)&0xfff"
//, &LARCH_opts.addrwidth_is_64, 0, 0},
{0, 0, "la.tls.le", "r,la",
"lu12i.w %1,%%tprel(%2)<<32>>44;"
"ori %1,%1,%%tprel(%2)&0xfff;"
"lu32i.d %1,%%tprel(%2)<<12>>44;"
"lu52i.d %1,%1,%%tprel(%2)>>52;"
, &LARCH_opts.addrwidth_is_64, 0, 0},

{0, 0, "la.tls.ie", "r,l",
"pcaddu12i %1,(%%pcrel(_GLOBAL_OFFSET_TABLE_+0x800)+%%tlsgot(%2))<<32>>44;"
"ld.w %1,%1,%%pcrel(_GLOBAL_OFFSET_TABLE_+4)+%%tlsgot(%2)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+4+0x800)+%%tlsgot(%2))>>12<<12);"
, &LARCH_opts.addrwidth_is_32, 0, 0},

{0, 0, "la.tls.ie", "r,l",
"pcaddu12i %1,(%%pcrel(_GLOBAL_OFFSET_TABLE_+0x800)+%%tlsgot(%2))>>12;"
"ld.d %1,%1,%%pcrel(_GLOBAL_OFFSET_TABLE_+4)+%%tlsgot(%2)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+4+0x800)+%%tlsgot(%2))>>12<<12);"
, &LARCH_opts.addrwidth_is_64, 0, 0},
{0, 0, "la.tls.ie", "r,r,l",
"pcaddu12i %1,(%%pcrel(_GLOBAL_OFFSET_TABLE_)+%%tlsgot(%3)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+0x80000000)+%%tlsgot(%3))>>32<<32))<<32>>44;"
"ori %2,$r0,(%%pcrel(_GLOBAL_OFFSET_TABLE_+4)+%%tlsgot(%3)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+4+0x80000000)+%%tlsgot(%3))>>32<<32))&0xfff;"
"lu32i.d %2,(%%pcrel(_GLOBAL_OFFSET_TABLE_+8+0x80000000)+%%tlsgot(%3))<<12>>44;"
"lu52i.d %2,%2,(%%pcrel(_GLOBAL_OFFSET_TABLE_+12+0x80000000)+%%tlsgot(%3))>>52;"
"ldx.d %1,%1,%2;"
, &LARCH_opts.addrwidth_is_64, 0, 0},

{0, 0, "la.tls.ld", "r,l", "la.tls.gd %1,%2", 0, 0, 0},
{0, 0, "la.tls.ld", "r,r,l",
"la.tls.gd %1,%2,%3"
, &LARCH_opts.addrwidth_is_64, 0, 0},


{0, 0, "la.tls.gd", "r,l",
"pcaddu12i %1,(%%pcrel(_GLOBAL_OFFSET_TABLE_+0x800)+%%tlsgd(%2))<<32>>44;"
"addi.w %1,%1,%%pcrel(_GLOBAL_OFFSET_TABLE_+4)+%%tlsgd(%2)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+4+0x800)+%%tlsgd(%2))>>12<<12);"
, &LARCH_opts.addrwidth_is_32, 0, 0},

{0, 0, "la.tls.gd", "r,l",
"pcaddu12i %1,(%%pcrel(_GLOBAL_OFFSET_TABLE_+0x800)+%%tlsgd(%2))>>12;"
"addi.d %1,%1,%%pcrel(_GLOBAL_OFFSET_TABLE_+4)+%%tlsgd(%2)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+4+0x800)+%%tlsgd(%2))>>12<<12);"
, &LARCH_opts.addrwidth_is_64, 0, 0},
{0, 0, "la.tls.gd", "r,r,l",
"pcaddu12i %1,(%%pcrel(_GLOBAL_OFFSET_TABLE_)+%%tlsgd(%3)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+0x80000000)+%%tlsgd(%3))>>32<<32))<<32>>44;"
"ori %2,$r0,(%%pcrel(_GLOBAL_OFFSET_TABLE_+4)+%%tlsgd(%3)-((%%pcrel(_GLOBAL_OFFSET_TABLE_+4+0x80000000)+%%tlsgd(%3))>>32<<32))&0xfff;"
"lu32i.d %2,(%%pcrel(_GLOBAL_OFFSET_TABLE_+8+0x80000000)+%%tlsgd(%3))<<12>>44;"
"lu52i.d %2,%2,(%%pcrel(_GLOBAL_OFFSET_TABLE_+12+0x80000000)+%%tlsgd(%3))>>52;"
"add.d %1,%1,%2;"
, &LARCH_opts.addrwidth_is_64, 0, 0},

{0} /* Terminate the list.  */
};

static struct loongarch_opcode loongarch_fix_opcodes[] = {
/* match,    mask,       name, format, macro, include, exclude, pinfo */
{0x00000800, 0xfffffc1c, "gr2scr", "cr0:2,r5:5", 0, 0, 0, 0},
{0x00000c00, 0xffffff80, "scr2gr", "r0:5,cr5:2", 0, 0, 0, 0},
{0x00001000, 0xfffffc00, "clo.w", "r0:5,r5:5", 0, 0, 0, 0},
{0x00001400, 0xfffffc00, "clz.w", "r0:5,r5:5", 0, 0, 0, 0},
{0x00001800, 0xfffffc00, "cto.w", "r0:5,r5:5", 0, 0, 0, 0},
{0x00001c00, 0xfffffc00, "ctz.w", "r0:5,r5:5", 0, 0, 0, 0},
{0x00002000, 0xfffffc00, "clo.d", "r0:5,r5:5", 0, 0, 0, 0},
{0x00002400, 0xfffffc00, "clz.d", "r0:5,r5:5", 0, 0, 0, 0},
{0x00002800, 0xfffffc00, "cto.d", "r0:5,r5:5", 0, 0, 0, 0},
{0x00002c00, 0xfffffc00, "ctz.d", "r0:5,r5:5", 0, 0, 0, 0},
{0x00003000, 0xfffffc00, "revb.2h", "r0:5,r5:5", 0, 0, 0, 0},
{0x00003400, 0xfffffc00, "revb.4h", "r0:5,r5:5", 0, 0, 0, 0},
{0x00003800, 0xfffffc00, "revb.2w", "r0:5,r5:5", 0, 0, 0, 0},
{0x00003c00, 0xfffffc00, "revb.d", "r0:5,r5:5", 0, 0, 0, 0},
{0x00004000, 0xfffffc00, "revh.2w", "r0:5,r5:5", 0, 0, 0, 0},
{0x00004400, 0xfffffc00, "revh.d", "r0:5,r5:5", 0, 0, 0, 0},
{0x00004800, 0xfffffc00, "bitrev.4b", "r0:5,r5:5", 0, 0, 0, 0},
{0x00004c00, 0xfffffc00, "bitrev.8b", "r0:5,r5:5", 0, 0, 0, 0},
{0x00005000, 0xfffffc00, "bitrev.w", "r0:5,r5:5", 0, 0, 0, 0},
{0x00005400, 0xfffffc00, "bitrev.d", "r0:5,r5:5", 0, 0, 0, 0},
{0x00005800, 0xfffffc00, "ext.w.h", "r0:5,r5:5", 0, 0, 0, 0},
{0x00005c00, 0xfffffc00, "ext.w.b", "r0:5,r5:5", 0, 0, 0, 0},
{0x00006000, 0xfffffc00, "rdtimel.w", "r0:5,r5:5", 0, 0, 0, 0},
{0x00006400, 0xfffffc00, "rdtimeh.w", "r0:5,r5:5", 0, 0, 0, 0},
{0x00006800, 0xfffffc00, "rdtime.d", "r0:5,r5:5", 0, 0, 0, 0},
{0x00006c00, 0xfffffc00, "cpucfg", "r0:5,r5:5", 0, 0, 0, 0},
{0x00007000, 0xffffff1f, "x86mttop", "u5:3", 0, 0, 0, 0},
{0x00007400, 0xffffffe0, "x86mftop", "r0:5", 0, 0, 0, 0},
{0x00007800, 0xfffffc00, "x86loope", "r0:5,r5:5", 0, 0, 0, 0},
{0x00007c00, 0xfffffc00, "x86loopne", "r0:5,r5:5", 0, 0, 0, 0},
{0x00008000, 0xfffffc1f, "x86inc.b", "r5:5", 0, 0, 0, 0},
{0x00008001, 0xfffffc1f, "x86inc.h", "r5:5", 0, 0, 0, 0},
{0x00008002, 0xfffffc1f, "x86inc.w", "r5:5", 0, 0, 0, 0},
{0x00008003, 0xfffffc1f, "x86inc.d", "r5:5", 0, 0, 0, 0},
{0x00008004, 0xfffffc1f, "x86dec.b", "r5:5", 0, 0, 0, 0},
{0x00008005, 0xfffffc1f, "x86dec.h", "r5:5", 0, 0, 0, 0},
{0x00008006, 0xfffffc1f, "x86dec.w", "r5:5", 0, 0, 0, 0},
{0x00008007, 0xfffffc1f, "x86dec.d", "r5:5", 0, 0, 0, 0},
{0x00008008, 0xffffffff, "x86settm", "", 0, 0, 0, 0},
{0x00008028, 0xffffffff, "x86clrtm", "", 0, 0, 0, 0},
{0x00008009, 0xffffffff, "x86inctop", "", 0, 0, 0, 0},
{0x00008029, 0xffffffff, "x86dectop", "", 0, 0, 0, 0},
{0x00010000, 0xffff801f, "asrtle.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x00018000, 0xffff801f, "asrtgt.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x00040000, 0xfffe0000, "alsl.w", "r0:5,r5:5,r10:5,u15:2+1", 0, 0, 0, 0},
{0x00060000, 0xfffe0000, "alsl.wu", "r0:5,r5:5,r10:5,u15:2+1", 0, 0, 0, 0},
{0x00080000, 0xfffe0000, "bytepick.w", "r0:5,r5:5,r10:5,u15:2", 0, 0, 0, 0},
{0x000c0000, 0xfffc0000, "bytepick.d", "r0:5,r5:5,r10:5,u15:3", 0, 0, 0, 0},
{0x00100000, 0xffff8000, "add.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00108000, 0xffff8000, "add.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00110000, 0xffff8000, "sub.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00118000, 0xffff8000, "sub.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00120000, 0xffff8000, "slt", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00128000, 0xffff8000, "sltu", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00130000, 0xffff8000, "maskeqz", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00138000, 0xffff8000, "masknez", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00140000, 0xffff8000, "nor", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00148000, 0xffff8000, "and", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00150000, 0xfffffc00, "move", "r0:5,r5:5", 0/* or %1,%2,$r0 */, 0, 0, 0},
{0x00150000, 0xffff8000, "or", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00158000, 0xffff8000, "xor", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00160000, 0xffff8000, "orn", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00168000, 0xffff8000, "andn", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00170000, 0xffff8000, "sll.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00178000, 0xffff8000, "srl.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00180000, 0xffff8000, "sra.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00188000, 0xffff8000, "sll.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00190000, 0xffff8000, "srl.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00198000, 0xffff8000, "sra.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001a0000, 0xffff8000, "rotr.b", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001a8000, 0xffff8000, "rotr.h", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001b0000, 0xffff8000, "rotr.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001b8000, 0xffff8000, "rotr.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001c0000, 0xffff8000, "mul.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001c8000, 0xffff8000, "mulh.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001d0000, 0xffff8000, "mulh.wu", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001d8000, 0xffff8000, "mul.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001e0000, 0xffff8000, "mulh.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001e8000, 0xffff8000, "mulh.du", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001f0000, 0xffff8000, "mulw.d.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x001f8000, 0xffff8000, "mulw.d.wu", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00200000, 0xffff8000, "div.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00208000, 0xffff8000, "mod.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00210000, 0xffff8000, "div.wu", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00218000, 0xffff8000, "mod.wu", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00220000, 0xffff8000, "div.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00228000, 0xffff8000, "mod.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00230000, 0xffff8000, "div.du", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00238000, 0xffff8000, "mod.du", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00240000, 0xffff8000, "crc.w.b.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00248000, 0xffff8000, "crc.w.h.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00250000, 0xffff8000, "crc.w.w.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00258000, 0xffff8000, "crc.w.d.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00260000, 0xffff8000, "crcc.w.b.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00268000, 0xffff8000, "crcc.w.h.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00270000, 0xffff8000, "crcc.w.w.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00278000, 0xffff8000, "crcc.w.d.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00290000, 0xffff8000, "addu12i.w", "r0:5,r5:5,s10:5", 0, 0, 0, 0},
{0x00298000, 0xffff8000, "addu12i.d", "r0:5,r5:5,s10:5", 0, 0, 0, 0},
{0x002a0000, 0xffff8000, "break", "u0:15", 0, 0, 0, 0},
{0x002a8000, 0xffff8000, "dbgcall", "u0:15", 0, 0, 0, 0},
{0x002b0000, 0xffff8000, "syscall", "u0:15", 0, 0, 0, 0},
{0x002b8000, 0xffff8000, "hypcall", "u0:15", 0, 0, 0, 0},
{0x002c0000, 0xfffe0000, "alsl.d", "r0:5,r5:5,r10:5,u15:2+1", 0, 0, 0, 0},
{0x00300000, 0xffff8000, "adc.b", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00308000, 0xffff8000, "adc.h", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00310000, 0xffff8000, "adc.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00318000, 0xffff8000, "adc.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00320000, 0xffff8000, "sbc.b", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00328000, 0xffff8000, "sbc.h", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00330000, 0xffff8000, "sbc.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00338000, 0xffff8000, "sbc.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00340000, 0xffff8000, "rcr.b", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00348000, 0xffff8000, "rcr.h", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00350000, 0xffff8000, "rcr.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00358000, 0xffff8000, "rcr.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x00364000, 0xffffc000, "armmove", "r0:5,r5:5,u10:4", 0, 0, 0, 0},
{0x00368000, 0xffffc3e0, "setx86j", "r0:5,u10:4", 0, 0, 0, 0},
{0x0036c000, 0xffffc3e0, "setarmj", "r0:5,u10:4", 0, 0, 0, 0},
{0x00370010, 0xffff8010, "armadd.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x00378010, 0xffff8010, "armsub.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x00380010, 0xffff8010, "armadc.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x00388010, 0xffff8010, "armsbc.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x00390010, 0xffff8010, "armand.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x00398010, 0xffff8010, "armor.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x003a0010, 0xffff8010, "armxor.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x003a8010, 0xffff8010, "armsll.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x003b0010, 0xffff8010, "armsrl.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x003b8010, 0xffff8010, "armsra.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x003c0010, 0xffff8010, "armrotr.w", "r5:5,r10:5,u0:4", 0, 0, 0, 0},
{0x003c8010, 0xffff8010, "armslli.w", "r5:5,u10:5,u0:4", 0, 0, 0, 0},
{0x003d0010, 0xffff8010, "armsrli.w", "r5:5,u10:5,u0:4", 0, 0, 0, 0},
{0x003d8010, 0xffff8010, "armsrai.w", "r5:5,u10:5,u0:4", 0, 0, 0, 0},
{0x003e0010, 0xffff8010, "armrotri.w", "r5:5,u10:5,u0:4", 0, 0, 0, 0},
{0x003e8000, 0xffff801f, "x86mul.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003e8001, 0xffff801f, "x86mul.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003e8002, 0xffff801f, "x86mul.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003e8003, 0xffff801f, "x86mul.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003e8004, 0xffff801f, "x86mul.bu", "r5:5,r10:5", 0, 0, 0, 0},
{0x003e8005, 0xffff801f, "x86mul.hu", "r5:5,r10:5", 0, 0, 0, 0},
{0x003e8006, 0xffff801f, "x86mul.wu", "r5:5,r10:5", 0, 0, 0, 0},
{0x003e8007, 0xffff801f, "x86mul.du", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0000, 0xffff801f, "x86add.wu", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0001, 0xffff801f, "x86add.du", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0002, 0xffff801f, "x86sub.wu", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0003, 0xffff801f, "x86sub.du", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0004, 0xffff801f, "x86add.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0005, 0xffff801f, "x86add.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0006, 0xffff801f, "x86add.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0007, 0xffff801f, "x86add.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0008, 0xffff801f, "x86sub.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0009, 0xffff801f, "x86sub.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f000a, 0xffff801f, "x86sub.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f000b, 0xffff801f, "x86sub.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f000c, 0xffff801f, "x86adc.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f000d, 0xffff801f, "x86adc.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f000e, 0xffff801f, "x86adc.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f000f, 0xffff801f, "x86adc.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0010, 0xffff801f, "x86sbc.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0011, 0xffff801f, "x86sbc.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0012, 0xffff801f, "x86sbc.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0013, 0xffff801f, "x86sbc.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0014, 0xffff801f, "x86sll.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0015, 0xffff801f, "x86sll.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0016, 0xffff801f, "x86sll.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0017, 0xffff801f, "x86sll.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0018, 0xffff801f, "x86srl.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f0019, 0xffff801f, "x86srl.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f001a, 0xffff801f, "x86srl.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f001b, 0xffff801f, "x86srl.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f001c, 0xffff801f, "x86sra.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f001d, 0xffff801f, "x86sra.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f001e, 0xffff801f, "x86sra.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f001f, 0xffff801f, "x86sra.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8000, 0xffff801f, "x86rotr.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8001, 0xffff801f, "x86rotr.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8002, 0xffff801f, "x86rotr.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8003, 0xffff801f, "x86rotr.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8004, 0xffff801f, "x86rotl.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8005, 0xffff801f, "x86rotl.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8006, 0xffff801f, "x86rotl.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8007, 0xffff801f, "x86rotl.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8008, 0xffff801f, "x86rcr.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8009, 0xffff801f, "x86rcr.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f800a, 0xffff801f, "x86rcr.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f800b, 0xffff801f, "x86rcr.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f800c, 0xffff801f, "x86rcl.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f800d, 0xffff801f, "x86rcl.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f800e, 0xffff801f, "x86rcl.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f800f, 0xffff801f, "x86rcl.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8010, 0xffff801f, "x86and.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8011, 0xffff801f, "x86and.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8012, 0xffff801f, "x86and.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8013, 0xffff801f, "x86and.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8014, 0xffff801f, "x86or.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8015, 0xffff801f, "x86or.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8016, 0xffff801f, "x86or.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8017, 0xffff801f, "x86or.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8018, 0xffff801f, "x86xor.b", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f8019, 0xffff801f, "x86xor.h", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f801a, 0xffff801f, "x86xor.w", "r5:5,r10:5", 0, 0, 0, 0},
{0x003f801b, 0xffff801f, "x86xor.d", "r5:5,r10:5", 0, 0, 0, 0},
{0x003fc01c, 0xffffc01f, "armnot.w", "r5:5,u10:4", 0, 0, 0, 0},
{0x003fc01d, 0xffffc01f, "armmov.w", "r5:5,u10:4", 0, 0, 0, 0},
{0x003fc01e, 0xffffc01f, "armmov.d", "r5:5,u10:4", 0, 0, 0, 0},
{0x003fc01f, 0xffffc01f, "armrrx.w", "r5:5,u10:4", 0, 0, 0, 0},
{0x00408000, 0xffff8000, "slli.w", "r0:5,r5:5,u10:5", 0, 0, 0, 0},
{0x00410000, 0xffff0000, "slli.d", "r0:5,r5:5,u10:6", 0, 0, 0, 0},
{0x00448000, 0xffff8000, "srli.w", "r0:5,r5:5,u10:5", 0, 0, 0, 0},
{0x00450000, 0xffff0000, "srli.d", "r0:5,r5:5,u10:6", 0, 0, 0, 0},
{0x00488000, 0xffff8000, "srai.w", "r0:5,r5:5,u10:5", 0, 0, 0, 0},
{0x00490000, 0xffff0000, "srai.d", "r0:5,r5:5,u10:6", 0, 0, 0, 0},
{0x004c2000, 0xffffe000, "rotri.b", "r0:5,r5:5,u10:3", 0, 0, 0, 0},
{0x004c4000, 0xffffc000, "rotri.h", "r0:5,r5:5,u10:4", 0, 0, 0, 0},
{0x004c8000, 0xffff8000, "rotri.w", "r0:5,r5:5,u10:5", 0, 0, 0, 0},
{0x004d0000, 0xffff0000, "rotri.d", "r0:5,r5:5,u10:6", 0, 0, 0, 0},
{0x00502000, 0xffffe000, "rcri.b", "r0:5,r5:5,u10:3", 0, 0, 0, 0},
{0x00504000, 0xffffc000, "rcri.h", "r0:5,r5:5,u10:4", 0, 0, 0, 0},
{0x00508000, 0xffff8000, "rcri.w", "r0:5,r5:5,u10:5", 0, 0, 0, 0},
{0x00510000, 0xffff0000, "rcri.d", "r0:5,r5:5,u10:6", 0, 0, 0, 0},
{0x00542000, 0xffffe01f, "x86slli.b", "r5:5,u10:3", 0, 0, 0, 0},
{0x00544001, 0xffffc01f, "x86slli.h", "r5:5,u10:4", 0, 0, 0, 0},
{0x00548002, 0xffff801f, "x86slli.w", "r5:5,u10:5", 0, 0, 0, 0},
{0x00550003, 0xffff001f, "x86slli.d", "r5:5,u10:6", 0, 0, 0, 0},
{0x00542004, 0xffffe01f, "x86srli.b", "r5:5,u10:3", 0, 0, 0, 0},
{0x00544005, 0xffffc01f, "x86srli.h", "r5:5,u10:4", 0, 0, 0, 0},
{0x00548006, 0xffff801f, "x86srli.w", "r5:5,u10:5", 0, 0, 0, 0},
{0x00550007, 0xffff001f, "x86srli.d", "r5:5,u10:6", 0, 0, 0, 0},
{0x00542008, 0xffffe01f, "x86srai.b", "r5:5,u10:3", 0, 0, 0, 0},
{0x00544009, 0xffffc01f, "x86srai.h", "r5:5,u10:4", 0, 0, 0, 0},
{0x0054800a, 0xffff801f, "x86srai.w", "r5:5,u10:5", 0, 0, 0, 0},
{0x0055000b, 0xffff001f, "x86srai.d", "r5:5,u10:6", 0, 0, 0, 0},
{0x0054200c, 0xffffe01f, "x86rotri.b", "r5:5,u10:3", 0, 0, 0, 0},
{0x0054400d, 0xffffc01f, "x86rotri.h", "r5:5,u10:4", 0, 0, 0, 0},
{0x0054800e, 0xffff801f, "x86rotri.w", "r5:5,u10:5", 0, 0, 0, 0},
{0x0055000f, 0xffff001f, "x86rotri.d", "r5:5,u10:6", 0, 0, 0, 0},
{0x00542010, 0xffffe01f, "x86rcri.b", "r5:5,u10:3", 0, 0, 0, 0},
{0x00544011, 0xffffc01f, "x86rcri.h", "r5:5,u10:4", 0, 0, 0, 0},
{0x00548012, 0xffff801f, "x86rcri.w", "r5:5,u10:5", 0, 0, 0, 0},
{0x00550013, 0xffff001f, "x86rcri.d", "r5:5,u10:6", 0, 0, 0, 0},
{0x00542014, 0xffffe01f, "x86rotli.b", "r5:5,u10:3", 0, 0, 0, 0},
{0x00544015, 0xffffc01f, "x86rotli.h", "r5:5,u10:4", 0, 0, 0, 0},
{0x00548016, 0xffff801f, "x86rotli.w", "r5:5,u10:5", 0, 0, 0, 0},
{0x00550017, 0xffff001f, "x86rotli.d", "r5:5,u10:6", 0, 0, 0, 0},
{0x00542018, 0xffffe01f, "x86rcli.b", "r5:5,u10:3", 0, 0, 0, 0},
{0x00544019, 0xffffc01f, "x86rcli.h", "r5:5,u10:4", 0, 0, 0, 0},
{0x0054801a, 0xffff801f, "x86rcli.w", "r5:5,u10:5", 0, 0, 0, 0},
{0x0055001b, 0xffff001f, "x86rcli.d", "r5:5,u10:6", 0, 0, 0, 0},
{0x00580000, 0xfffc0000, "x86settag", "r0:5,u5:5,u10:8", 0, 0, 0, 0},
{0x005c0000, 0xfffc03e0, "x86mfflag", "r0:5,u10:8", 0, 0, 0, 0},
{0x005c0020, 0xfffc03e0, "x86mtflag", "r0:5,u10:8", 0, 0, 0, 0},
{0x005c0040, 0xfffc03e0, "armmfflag", "r0:5,u10:8", 0, 0, 0, 0},
{0x005c0060, 0xfffc03e0, "armmtflag", "r0:5,u10:8", 0, 0, 0, 0},
{0x00600000, 0xffe08000, "bstrins.w", "r0:5,r5:5,u16:5,u10:5", 0, 0, 0, 0},
{0x00608000, 0xffe08000, "bstrpick.w", "r0:5,r5:5,u16:5,u10:5", 0, 0, 0, 0},
{0x00800000, 0xffc00000, "bstrins.d", "r0:5,r5:5,u16:6,u10:6", 0, 0, 0, 0},
{0x00c00000, 0xffc00000, "bstrpick.d", "r0:5,r5:5,u16:6,u10:6", 0, 0, 0, 0},
{0} /* Terminate the list.  */
};

static struct loongarch_opcode loongarch_float_opcodes[] = {
/* match,    mask,       name, format, macro, include, exclude, pinfo */
{0x01000000, 0xffff8000, "fadd.h", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01008000, 0xffff8000, "fadd.s", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01010000, 0xffff8000, "fadd.d", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01020000, 0xffff8000, "fsub.h", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01028000, 0xffff8000, "fsub.s", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01030000, 0xffff8000, "fsub.d", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01040000, 0xffff8000, "fmul.h", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01048000, 0xffff8000, "fmul.s", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01050000, 0xffff8000, "fmul.d", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01060000, 0xffff8000, "fdiv.h", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01068000, 0xffff8000, "fdiv.s", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01070000, 0xffff8000, "fdiv.d", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01080000, 0xffff8000, "fmax.h", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01088000, 0xffff8000, "fmax.s", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01090000, 0xffff8000, "fmax.d", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x010a0000, 0xffff8000, "fmin.h", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x010a8000, 0xffff8000, "fmin.s", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x010b0000, 0xffff8000, "fmin.d", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x010c0000, 0xffff8000, "fmaxa.h", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x010c8000, 0xffff8000, "fmaxa.s", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x010d0000, 0xffff8000, "fmaxa.d", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x010e0000, 0xffff8000, "fmina.h", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x010e8000, 0xffff8000, "fmina.s", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x010f0000, 0xffff8000, "fmina.d", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01100000, 0xffff8000, "fscaleb.h", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01108000, 0xffff8000, "fscaleb.s", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01110000, 0xffff8000, "fscaleb.d", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01120000, 0xffff8000, "fcopysign.h", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01128000, 0xffff8000, "fcopysign.s", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01130000, 0xffff8000, "fcopysign.d", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01140000, 0xfffffc00, "fabs.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01140400, 0xfffffc00, "fabs.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01140800, 0xfffffc00, "fabs.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01141000, 0xfffffc00, "fneg.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01141400, 0xfffffc00, "fneg.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01141800, 0xfffffc00, "fneg.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01142000, 0xfffffc00, "flogb.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01142400, 0xfffffc00, "flogb.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01142800, 0xfffffc00, "flogb.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01143000, 0xfffffc00, "fclass.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01143400, 0xfffffc00, "fclass.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01143800, 0xfffffc00, "fclass.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01144000, 0xfffffc00, "fsqrt.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01144400, 0xfffffc00, "fsqrt.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01144800, 0xfffffc00, "fsqrt.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01145000, 0xfffffc00, "frecip.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01145400, 0xfffffc00, "frecip.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01145800, 0xfffffc00, "frecip.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01146000, 0xfffffc00, "frsqrt.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01146400, 0xfffffc00, "frsqrt.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01146800, 0xfffffc00, "frsqrt.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01147000, 0xfffffc00, "frecipe.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01147400, 0xfffffc00, "frecipe.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01147800, 0xfffffc00, "frecipe.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01148000, 0xfffffc00, "frsqrte.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01148400, 0xfffffc00, "frsqrte.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01148800, 0xfffffc00, "frsqrte.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01149000, 0xfffffc00, "fmov.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01149400, 0xfffffc00, "fmov.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01149800, 0xfffffc00, "fmov.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x0114a000, 0xfffffc00, "movgr2fr.h", "f0:5,r5:5", 0, 0, 0, 0},
{0x0114a400, 0xfffffc00, "movgr2fr.w", "f0:5,r5:5", 0, 0, 0, 0},
{0x0114a800, 0xfffffc00, "movgr2fr.d", "f0:5,r5:5", 0, 0, 0, 0},
{0x0114ac00, 0xfffffc00, "movgr2frh.w", "f0:5,r5:5", 0, 0, 0, 0},
{0x0114b000, 0xfffffc00, "movfr2gr.h", "r0:5,f5:5", 0, 0, 0, 0},
{0x0114b400, 0xfffffc00, "movfr2gr.s", "r0:5,f5:5", 0, 0, 0, 0},
{0x0114b800, 0xfffffc00, "movfr2gr.d", "r0:5,f5:5", 0, 0, 0, 0},
{0x0114bc00, 0xfffffc00, "movfrh2gr.s", "r0:5,f5:5", 0, 0, 0, 0},
{0x0114c000, 0xfffffc00, "movgr2fcsr", "r0:5,r5:5", 0, 0, 0, 0},
{0x0114c800, 0xfffffc00, "movfcsr2gr", "r0:5,r5:5", 0, 0, 0, 0},
{0x0114d000, 0xfffffc18, "movfr2cf", "c0:3,f5:5", 0, 0, 0, 0},
{0x0114d400, 0xffffff00, "movcf2fr", "f0:5,c5:3", 0, 0, 0, 0},
{0x0114d800, 0xfffffc18, "movgr2cf", "c0:3,r5:5", 0, 0, 0, 0},
{0x0114dc00, 0xffffff00, "movcf2gr", "r0:5,c5:3", 0, 0, 0, 0},
{0x0114e000, 0xfffffc00, "fcvt.ld.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x0114e400, 0xfffffc00, "fcvt.ud.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01150000, 0xffff8000, "fcvt.d.ld", "f0:5,f5:5,f10:5", 0, 0, 0, 0},
{0x01180400, 0xfffffc00, "fcvtrm.h.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01180800, 0xfffffc00, "fcvtrm.h.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01180c00, 0xfffffc00, "fcvtrm.h.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x01181000, 0xfffffc00, "fcvtrm.s.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01181800, 0xfffffc00, "fcvtrm.s.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01181c00, 0xfffffc00, "fcvtrm.s.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x01182000, 0xfffffc00, "fcvtrm.d.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01182400, 0xfffffc00, "fcvtrm.d.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01182c00, 0xfffffc00, "fcvtrm.d.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x01183000, 0xfffffc00, "fcvtrm.q.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01183400, 0xfffffc00, "fcvtrm.q.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01183800, 0xfffffc00, "fcvtrm.q.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01184400, 0xfffffc00, "fcvtrp.h.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01184800, 0xfffffc00, "fcvtrp.h.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01184c00, 0xfffffc00, "fcvtrp.h.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x01185000, 0xfffffc00, "fcvtrp.s.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01185800, 0xfffffc00, "fcvtrp.s.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01185c00, 0xfffffc00, "fcvtrp.s.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x01186000, 0xfffffc00, "fcvtrp.d.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01186400, 0xfffffc00, "fcvtrp.d.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01186c00, 0xfffffc00, "fcvtrp.d.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x01187000, 0xfffffc00, "fcvtrp.q.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01187400, 0xfffffc00, "fcvtrp.q.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01187800, 0xfffffc00, "fcvtrp.q.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01188400, 0xfffffc00, "fcvtrz.h.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01188800, 0xfffffc00, "fcvtrz.h.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01188c00, 0xfffffc00, "fcvtrz.h.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x01189000, 0xfffffc00, "fcvtrz.s.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01189800, 0xfffffc00, "fcvtrz.s.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01189c00, 0xfffffc00, "fcvtrz.s.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118a000, 0xfffffc00, "fcvtrz.d.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118a400, 0xfffffc00, "fcvtrz.d.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118ac00, 0xfffffc00, "fcvtrz.d.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118b000, 0xfffffc00, "fcvtrz.q.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118b400, 0xfffffc00, "fcvtrz.q.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118b800, 0xfffffc00, "fcvtrz.q.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118c400, 0xfffffc00, "fcvtrne.h.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118c800, 0xfffffc00, "fcvtrne.h.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118cc00, 0xfffffc00, "fcvtrne.h.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118d000, 0xfffffc00, "fcvtrne.s.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118d800, 0xfffffc00, "fcvtrne.s.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118dc00, 0xfffffc00, "fcvtrne.s.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118e000, 0xfffffc00, "fcvtrne.d.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118e400, 0xfffffc00, "fcvtrne.d.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118ec00, 0xfffffc00, "fcvtrne.d.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118f000, 0xfffffc00, "fcvtrne.q.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118f400, 0xfffffc00, "fcvtrne.q.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x0118f800, 0xfffffc00, "fcvtrne.q.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01190400, 0xfffffc00, "fcvt.h.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01190800, 0xfffffc00, "fcvt.h.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01190c00, 0xfffffc00, "fcvt.h.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x01191000, 0xfffffc00, "fcvt.s.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01191800, 0xfffffc00, "fcvt.s.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x01191c00, 0xfffffc00, "fcvt.s.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x01192000, 0xfffffc00, "fcvt.d.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01192400, 0xfffffc00, "fcvt.d.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01192c00, 0xfffffc00, "fcvt.d.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x01193000, 0xfffffc00, "fcvt.q.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x01193400, 0xfffffc00, "fcvt.q.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x01193800, 0xfffffc00, "fcvt.q.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a0000, 0xfffffc00, "ftintrm.w.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a0400, 0xfffffc00, "ftintrm.w.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a0800, 0xfffffc00, "ftintrm.w.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a0c00, 0xfffffc00, "ftintrm.w.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a1000, 0xfffffc00, "ftintrm.wu.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a1400, 0xfffffc00, "ftintrm.wu.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a1800, 0xfffffc00, "ftintrm.wu.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a1c00, 0xfffffc00, "ftintrm.wu.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a2000, 0xfffffc00, "ftintrm.l.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a2400, 0xfffffc00, "ftintrm.l.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a2800, 0xfffffc00, "ftintrm.l.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a2c00, 0xfffffc00, "ftintrm.l.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a3000, 0xfffffc00, "ftintrm.lu.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a3400, 0xfffffc00, "ftintrm.lu.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a3800, 0xfffffc00, "ftintrm.lu.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a3c00, 0xfffffc00, "ftintrm.lu.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a4000, 0xfffffc00, "ftintrp.w.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a4400, 0xfffffc00, "ftintrp.w.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a4800, 0xfffffc00, "ftintrp.w.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a4c00, 0xfffffc00, "ftintrp.w.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a5000, 0xfffffc00, "ftintrp.wu.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a5400, 0xfffffc00, "ftintrp.wu.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a5800, 0xfffffc00, "ftintrp.wu.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a5c00, 0xfffffc00, "ftintrp.wu.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a6000, 0xfffffc00, "ftintrp.l.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a6400, 0xfffffc00, "ftintrp.l.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a6800, 0xfffffc00, "ftintrp.l.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a6c00, 0xfffffc00, "ftintrp.l.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a7000, 0xfffffc00, "ftintrp.lu.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a7400, 0xfffffc00, "ftintrp.lu.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a7800, 0xfffffc00, "ftintrp.lu.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a7c00, 0xfffffc00, "ftintrp.lu.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a8000, 0xfffffc00, "ftintrz.w.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a8400, 0xfffffc00, "ftintrz.w.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a8800, 0xfffffc00, "ftintrz.w.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a8c00, 0xfffffc00, "ftintrz.w.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a9000, 0xfffffc00, "ftintrz.wu.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a9400, 0xfffffc00, "ftintrz.wu.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a9800, 0xfffffc00, "ftintrz.wu.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011a9c00, 0xfffffc00, "ftintrz.wu.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011aa000, 0xfffffc00, "ftintrz.l.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011aa400, 0xfffffc00, "ftintrz.l.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011aa800, 0xfffffc00, "ftintrz.l.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011aac00, 0xfffffc00, "ftintrz.l.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ab000, 0xfffffc00, "ftintrz.lu.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ab400, 0xfffffc00, "ftintrz.lu.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ab800, 0xfffffc00, "ftintrz.lu.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011abc00, 0xfffffc00, "ftintrz.lu.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ac000, 0xfffffc00, "ftintrne.w.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ac400, 0xfffffc00, "ftintrne.w.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ac800, 0xfffffc00, "ftintrne.w.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011acc00, 0xfffffc00, "ftintrne.w.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ad000, 0xfffffc00, "ftintrne.wu.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ad400, 0xfffffc00, "ftintrne.wu.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ad800, 0xfffffc00, "ftintrne.wu.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011adc00, 0xfffffc00, "ftintrne.wu.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ae000, 0xfffffc00, "ftintrne.l.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ae400, 0xfffffc00, "ftintrne.l.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ae800, 0xfffffc00, "ftintrne.l.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011aec00, 0xfffffc00, "ftintrne.l.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011af000, 0xfffffc00, "ftintrne.lu.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011af400, 0xfffffc00, "ftintrne.lu.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011af800, 0xfffffc00, "ftintrne.lu.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011afc00, 0xfffffc00, "ftintrne.lu.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b0000, 0xfffffc00, "ftint.w.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b0400, 0xfffffc00, "ftint.w.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b0800, 0xfffffc00, "ftint.w.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b0c00, 0xfffffc00, "ftint.w.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b1000, 0xfffffc00, "ftint.wu.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b1400, 0xfffffc00, "ftint.wu.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b1800, 0xfffffc00, "ftint.wu.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b1c00, 0xfffffc00, "ftint.wu.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b2000, 0xfffffc00, "ftint.l.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b2400, 0xfffffc00, "ftint.l.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b2800, 0xfffffc00, "ftint.l.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b2c00, 0xfffffc00, "ftint.l.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b3000, 0xfffffc00, "ftint.lu.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b3400, 0xfffffc00, "ftint.lu.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b3800, 0xfffffc00, "ftint.lu.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011b3c00, 0xfffffc00, "ftint.lu.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c0000, 0xfffffc00, "ffintrm.h.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c0400, 0xfffffc00, "ffintrm.h.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c0800, 0xfffffc00, "ffintrm.h.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c0c00, 0xfffffc00, "ffintrm.h.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c1000, 0xfffffc00, "ffintrm.s.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c1400, 0xfffffc00, "ffintrm.s.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c1800, 0xfffffc00, "ffintrm.s.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c1c00, 0xfffffc00, "ffintrm.s.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c2000, 0xfffffc00, "ffintrm.d.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c2400, 0xfffffc00, "ffintrm.d.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c2800, 0xfffffc00, "ffintrm.d.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c2c00, 0xfffffc00, "ffintrm.d.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c3000, 0xfffffc00, "ffintrm.q.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c3400, 0xfffffc00, "ffintrm.q.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c3800, 0xfffffc00, "ffintrm.q.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c3c00, 0xfffffc00, "ffintrm.q.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c4000, 0xfffffc00, "ffintrp.h.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c4400, 0xfffffc00, "ffintrp.h.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c4800, 0xfffffc00, "ffintrp.h.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c4c00, 0xfffffc00, "ffintrp.h.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c5000, 0xfffffc00, "ffintrp.s.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c5400, 0xfffffc00, "ffintrp.s.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c5800, 0xfffffc00, "ffintrp.s.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c5c00, 0xfffffc00, "ffintrp.s.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c6000, 0xfffffc00, "ffintrp.d.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c6400, 0xfffffc00, "ffintrp.d.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c6800, 0xfffffc00, "ffintrp.d.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c6c00, 0xfffffc00, "ffintrp.d.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c7000, 0xfffffc00, "ffintrp.q.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c7400, 0xfffffc00, "ffintrp.q.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c7800, 0xfffffc00, "ffintrp.q.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c7c00, 0xfffffc00, "ffintrp.q.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c8000, 0xfffffc00, "ffintrz.h.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c8400, 0xfffffc00, "ffintrz.h.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c8800, 0xfffffc00, "ffintrz.h.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c8c00, 0xfffffc00, "ffintrz.h.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c9000, 0xfffffc00, "ffintrz.s.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c9400, 0xfffffc00, "ffintrz.s.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c9800, 0xfffffc00, "ffintrz.s.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011c9c00, 0xfffffc00, "ffintrz.s.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ca000, 0xfffffc00, "ffintrz.d.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ca400, 0xfffffc00, "ffintrz.d.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ca800, 0xfffffc00, "ffintrz.d.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cac00, 0xfffffc00, "ffintrz.d.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cb000, 0xfffffc00, "ffintrz.q.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cb400, 0xfffffc00, "ffintrz.q.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cb800, 0xfffffc00, "ffintrz.q.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cbc00, 0xfffffc00, "ffintrz.q.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cc000, 0xfffffc00, "ffintrne.h.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cc400, 0xfffffc00, "ffintrne.h.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cc800, 0xfffffc00, "ffintrne.h.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ccc00, 0xfffffc00, "ffintrne.h.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cd000, 0xfffffc00, "ffintrne.s.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cd400, 0xfffffc00, "ffintrne.s.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cd800, 0xfffffc00, "ffintrne.s.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cdc00, 0xfffffc00, "ffintrne.s.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ce000, 0xfffffc00, "ffintrne.d.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ce400, 0xfffffc00, "ffintrne.d.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011ce800, 0xfffffc00, "ffintrne.d.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cec00, 0xfffffc00, "ffintrne.d.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cf000, 0xfffffc00, "ffintrne.q.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cf400, 0xfffffc00, "ffintrne.q.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cf800, 0xfffffc00, "ffintrne.q.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011cfc00, 0xfffffc00, "ffintrne.q.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d0000, 0xfffffc00, "ffint.h.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d0400, 0xfffffc00, "ffint.h.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d0800, 0xfffffc00, "ffint.h.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d0c00, 0xfffffc00, "ffint.h.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d1000, 0xfffffc00, "ffint.s.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d1400, 0xfffffc00, "ffint.s.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d1800, 0xfffffc00, "ffint.s.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d1c00, 0xfffffc00, "ffint.s.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d2000, 0xfffffc00, "ffint.d.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d2400, 0xfffffc00, "ffint.d.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d2800, 0xfffffc00, "ffint.d.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d2c00, 0xfffffc00, "ffint.d.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d3000, 0xfffffc00, "ffint.q.w", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d3400, 0xfffffc00, "ffint.q.wu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d3800, 0xfffffc00, "ffint.q.l", "f0:5,f5:5", 0, 0, 0, 0},
{0x011d3c00, 0xfffffc00, "ffint.q.lu", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e0000, 0xfffffc00, "frintrm.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e0400, 0xfffffc00, "frintrm.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e0800, 0xfffffc00, "frintrm.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e0c00, 0xfffffc00, "frintrm.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e1000, 0xfffffc00, "frintrp.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e1400, 0xfffffc00, "frintrp.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e1800, 0xfffffc00, "frintrp.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e1c00, 0xfffffc00, "frintrp.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e2000, 0xfffffc00, "frintrz.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e2400, 0xfffffc00, "frintrz.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e2800, 0xfffffc00, "frintrz.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e2c00, 0xfffffc00, "frintrz.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e3000, 0xfffffc00, "frintrne.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e3400, 0xfffffc00, "frintrne.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e3800, 0xfffffc00, "frintrne.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e3c00, 0xfffffc00, "frintrne.q", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e4000, 0xfffffc00, "frint.h", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e4400, 0xfffffc00, "frint.s", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e4800, 0xfffffc00, "frint.d", "f0:5,f5:5", 0, 0, 0, 0},
{0x011e4c00, 0xfffffc00, "frint.q", "f0:5,f5:5", 0, 0, 0, 0},
{0} /* Terminate the list.  */
};

static struct loongarch_opcode loongarch_lmm_opcodes[] = {
/* match,    mask,       name, format, macro, include, exclude, pinfo */
{0x02000000, 0xffc00000, "slti", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x02400000, 0xffc00000, "sltui", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x02800000, 0xffc00000, "addi.w", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x02c00000, 0xffc00000, "addi.d", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x03000000, 0xffc00000, "lu52i.d", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0, 0, "nop", "", "andi $r0,$r0,0", 0, 0, 0},
{0x03400000, 0xffc00000, "andi", "r0:5,r5:5,u10:12", 0, 0, 0, 0},
{0x03800000, 0xffc00000, "ori", "r0:5,r5:5,u10:12", 0, 0, 0, 0},
{0x03c00000, 0xffc00000, "xori", "r0:5,r5:5,u10:12", 0, 0, 0, 0},
{0x10000000, 0xfc000000, "addu16i.d", "r0:5,r5:5,s10:16", 0, 0, 0, 0},
{0x14000000, 0xfe000000, "lu12i.w", "r0:5,s5:20", 0, 0, 0, 0},
{0x16000000, 0xfe000000, "lu32i.d", "r0:5,s5:20", 0, 0, 0, 0},
{0x18000000, 0xfe000000, "pcaddi", "r0:5,s5:20", 0, 0, 0, 0},
{0x1a000000, 0xfe000000, "pcalau12i", "r0:5,s5:20", 0, 0, 0, 0},
{0x1c000000, 0xfe000000, "pcaddu12i", "r0:5,s5:20", 0, 0, 0, 0},
{0x1e000000, 0xfe000000, "pcaddu18i", "r0:5,s5:20", 0, 0, 0, 0},
{0} /* Terminate the list.  */
};

static struct loongarch_opcode loongarch_privilege_opcodes[] = {
/* match,    mask,       name, format, macro, include, exclude, pinfo */
{0x04000000, 0xff0003e0, "csrrd", "r0:5,u10:14", 0, 0, 0, 0},
{0x04000020, 0xff0003e0, "csrwr", "r0:5,u10:14", 0, 0, 0, 0},
{0x04000000, 0xff000000, "csrxchg", "r0:5,r5:5,u10:14", 0, 0, 0, 0},
{0x05000000, 0xff0003e0, "gcsrrd", "r0:5,u10:14", 0, 0, 0, 0},
{0x05000020, 0xff0003e0, "gcsrwr", "r0:5,u10:14", 0, 0, 0, 0},
{0x05000000, 0xff000000, "gcsrxchg", "r0:5,r5:5,u10:14", 0, 0, 0, 0},
{0x06000000, 0xffc00000, "cache", "u0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x06400000, 0xfffc0000, "lddir", "r0:5,r5:5,u10:8", 0, 0, 0, 0},
{0x06440000, 0xfffc001f, "ldpte", "r5:5,u10:8", 0, 0, 0, 0},
{0x06480000, 0xfffffc00, "iocsrrd.b", "r0:5,r5:5", 0, 0, 0, 0},
{0x06480400, 0xfffffc00, "iocsrrd.h", "r0:5,r5:5", 0, 0, 0, 0},
{0x06480800, 0xfffffc00, "iocsrrd.w", "r0:5,r5:5", 0, 0, 0, 0},
{0x06480c00, 0xfffffc00, "iocsrrd.d", "r0:5,r5:5", 0, 0, 0, 0},
{0x06481000, 0xfffffc00, "iocsrwr.b", "r0:5,r5:5", 0, 0, 0, 0},
{0x06481400, 0xfffffc00, "iocsrwr.h", "r0:5,r5:5", 0, 0, 0, 0},
{0x06481800, 0xfffffc00, "iocsrwr.w", "r0:5,r5:5", 0, 0, 0, 0},
{0x06481c00, 0xfffffc00, "iocsrwr.d", "r0:5,r5:5", 0, 0, 0, 0},
{0x06482000, 0xffffffff, "tlbinv", "", 0, 0, 0, 0},
{0x06482001, 0xffffffff, "gtlbinv", "", 0, 0, 0, 0},
{0x06482400, 0xffffffff, "tlbflush", "", 0, 0, 0, 0},
{0x06482401, 0xffffffff, "gtlbflush", "", 0, 0, 0, 0},
{0x06482800, 0xffffffff, "tlbp", "", 0, 0, 0, 0},
{0x06482801, 0xffffffff, "gtlbp", "", 0, 0, 0, 0},
{0x06482c00, 0xffffffff, "tlbr", "", 0, 0, 0, 0},
{0x06482c01, 0xffffffff, "gtlbr", "", 0, 0, 0, 0},
{0x06483000, 0xffffffff, "tlbwi", "", 0, 0, 0, 0},
{0x06483001, 0xffffffff, "gtlbwi", "", 0, 0, 0, 0},
{0x06483400, 0xffffffff, "tlbwr", "", 0, 0, 0, 0},
{0x06483401, 0xffffffff, "gtlbwr", "", 0, 0, 0, 0},
{0x06483800, 0xffffffff, "eret", "", 0, 0, 0, 0},
{0x06483c00, 0xffffffff, "deret", "", 0, 0, 0, 0},
{0x06488000, 0xffff8000, "wait", "u0:15", 0, 0, 0, 0},
{0x06498000, 0xffff8000, "invtlb", "u0:5,r5:5,r10:5", 0, 0, 0, 0},
{0} /* Terminate the list.  */
};

static struct loongarch_opcode loongarch_4opt_opcodes[] = {
/* match,    mask,       name, format, macro, include, exclude, pinfo */
{0x08000000, 0xfff00000, "fmadd.h", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08100000, 0xfff00000, "fmadd.s", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08200000, 0xfff00000, "fmadd.d", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08400000, 0xfff00000, "fmsub.h", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08500000, 0xfff00000, "fmsub.s", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08600000, 0xfff00000, "fmsub.d", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08800000, 0xfff00000, "fnmadd.h", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08900000, 0xfff00000, "fnmadd.s", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08a00000, 0xfff00000, "fnmadd.d", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08c00000, 0xfff00000, "fnmsub.h", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08d00000, 0xfff00000, "fnmsub.s", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x08e00000, 0xfff00000, "fnmsub.d", "f0:5,f5:5,f10:5,f15:5", 0, 0, 0, 0},
{0x0c000000, 0xffff8018, "fcmp.caf.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c008000, 0xffff8018, "fcmp.saf.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c010000, 0xffff8018, "fcmp.clt.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c018000, 0xffff8018, "fcmp.slt.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c020000, 0xffff8018, "fcmp.ceq.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c028000, 0xffff8018, "fcmp.seq.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c030000, 0xffff8018, "fcmp.cle.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c038000, 0xffff8018, "fcmp.sle.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c040000, 0xffff8018, "fcmp.cun.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c048000, 0xffff8018, "fcmp.sun.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c050000, 0xffff8018, "fcmp.cult.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c058000, 0xffff8018, "fcmp.sult.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c060000, 0xffff8018, "fcmp.cueq.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c068000, 0xffff8018, "fcmp.sueq.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c070000, 0xffff8018, "fcmp.cule.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c078000, 0xffff8018, "fcmp.sule.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c080000, 0xffff8018, "fcmp.cne.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c088000, 0xffff8018, "fcmp.sne.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c0a0000, 0xffff8018, "fcmp.cor.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c0a8000, 0xffff8018, "fcmp.sor.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c0c0000, 0xffff8018, "fcmp.cune.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c0c8000, 0xffff8018, "fcmp.sune.h", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c100000, 0xffff8018, "fcmp.caf.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c108000, 0xffff8018, "fcmp.saf.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c110000, 0xffff8018, "fcmp.clt.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c118000, 0xffff8018, "fcmp.slt.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c120000, 0xffff8018, "fcmp.ceq.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c128000, 0xffff8018, "fcmp.seq.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c130000, 0xffff8018, "fcmp.cle.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c138000, 0xffff8018, "fcmp.sle.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c140000, 0xffff8018, "fcmp.cun.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c148000, 0xffff8018, "fcmp.sun.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c150000, 0xffff8018, "fcmp.cult.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c158000, 0xffff8018, "fcmp.sult.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c160000, 0xffff8018, "fcmp.cueq.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c168000, 0xffff8018, "fcmp.sueq.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c170000, 0xffff8018, "fcmp.cule.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c178000, 0xffff8018, "fcmp.sule.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c180000, 0xffff8018, "fcmp.cne.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c188000, 0xffff8018, "fcmp.sne.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c1a0000, 0xffff8018, "fcmp.cor.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c1a8000, 0xffff8018, "fcmp.sor.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c1c0000, 0xffff8018, "fcmp.cune.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c1c8000, 0xffff8018, "fcmp.sune.s", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c200000, 0xffff8018, "fcmp.caf.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c208000, 0xffff8018, "fcmp.saf.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c210000, 0xffff8018, "fcmp.clt.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c218000, 0xffff8018, "fcmp.slt.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c220000, 0xffff8018, "fcmp.ceq.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c228000, 0xffff8018, "fcmp.seq.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c230000, 0xffff8018, "fcmp.cle.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c238000, 0xffff8018, "fcmp.sle.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c240000, 0xffff8018, "fcmp.cun.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c248000, 0xffff8018, "fcmp.sun.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c250000, 0xffff8018, "fcmp.cult.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c258000, 0xffff8018, "fcmp.sult.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c260000, 0xffff8018, "fcmp.cueq.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c268000, 0xffff8018, "fcmp.sueq.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c270000, 0xffff8018, "fcmp.cule.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c278000, 0xffff8018, "fcmp.sule.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c280000, 0xffff8018, "fcmp.cne.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c288000, 0xffff8018, "fcmp.sne.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c2a0000, 0xffff8018, "fcmp.cor.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c2a8000, 0xffff8018, "fcmp.sor.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c2c0000, 0xffff8018, "fcmp.cune.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0c2c8000, 0xffff8018, "fcmp.sune.d", "c0:3,f5:5,f10:5", 0, 0, 0, 0},
{0x0d000000, 0xfffc0000, "fsel", "f0:5,f5:5,f10:5,c15:3", 0, 0, 0, 0},
{0} /* Terminate the list.  */
};

static struct loongarch_opcode loongarch_load_store_opcodes[] = {
/* match,    mask,       name, format, macro, include, exclude, pinfo */
{0x20000000, 0xff000000, "ll.w", "r0:5,r5:5,s10:14<<2", 0, 0, 0, 0},
{0x21000000, 0xff000000, "sc.w", "r0:5,r5:5,s10:14<<2", 0, 0, 0, 0},
{0x22000000, 0xff000000, "ll.d", "r0:5,r5:5,s10:14<<2", 0, 0, 0, 0},
{0x23000000, 0xff000000, "sc.d", "r0:5,r5:5,s10:14<<2", 0, 0, 0, 0},
{0x24000000, 0xff000000, "ldptr.w", "r0:5,r5:5,s10:14<<2", 0, 0, 0, 0},
{0x25000000, 0xff000000, "stptr.w", "r0:5,r5:5,s10:14<<2", 0, 0, 0, 0},
{0x26000000, 0xff000000, "ldptr.d", "r0:5,r5:5,s10:14<<2", 0, 0, 0, 0},
{0x27000000, 0xff000000, "stptr.d", "r0:5,r5:5,s10:14<<2", 0, 0, 0, 0},
{0x28000000, 0xffc00000, "ld.b", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x28400000, 0xffc00000, "ld.h", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x28800000, 0xffc00000, "ld.w", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x28c00000, 0xffc00000, "ld.d", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x29000000, 0xffc00000, "st.b", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x29400000, 0xffc00000, "st.h", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x29800000, 0xffc00000, "st.w", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x29c00000, 0xffc00000, "st.d", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2a000000, 0xffc00000, "ld.bu", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2a400000, 0xffc00000, "ld.hu", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2a800000, 0xffc00000, "ld.wu", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2ac00000, 0xffc00000, "preld", "u0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2b000000, 0xffc00000, "fld.s", "f0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2b400000, 0xffc00000, "fst.s", "f0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2b800000, 0xffc00000, "fld.d", "f0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2bc00000, 0xffc00000, "fst.d", "f0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2e000000, 0xffc00000, "ldl.w", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2e400000, 0xffc00000, "ldr.w", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2e800000, 0xffc00000, "ldl.d", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2ec00000, 0xffc00000, "ldr.d", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2f000000, 0xffc00000, "stl.w", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2f400000, 0xffc00000, "str.w", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2f800000, 0xffc00000, "stl.d", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x2fc00000, 0xffc00000, "str.d", "r0:5,r5:5,s10:12", 0, 0, 0, 0},
{0x38000000, 0xffff8000, "ldx.b", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38040000, 0xffff8000, "ldx.h", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38080000, 0xffff8000, "ldx.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x380c0000, 0xffff8000, "ldx.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38100000, 0xffff8000, "stx.b", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38140000, 0xffff8000, "stx.h", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38180000, 0xffff8000, "stx.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x381c0000, 0xffff8000, "stx.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38200000, 0xffff8000, "ldx.bu", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38240000, 0xffff8000, "ldx.hu", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38280000, 0xffff8000, "ldx.wu", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x382c0000, 0xffff8000, "preldx", "u0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38300000, 0xffff8000, "fldx.s", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38340000, 0xffff8000, "fldx.d", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38380000, 0xffff8000, "fstx.s", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x383c0000, 0xffff8000, "fstx.d", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0, 0, "amswap.w", "r,r,r,u0:0", "amswap.w %1,%2,%3", 0, 0, 0},
{0x38600000, 0xffff8000, "amswap.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amswap.d", "r,r,r,u0:0", "amswap.d %1,%2,%3", 0, 0, 0},
{0x38608000, 0xffff8000, "amswap.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amadd.w", "r,r,r,u0:0", "amadd.w %1,%2,%3", 0, 0, 0},
{0x38610000, 0xffff8000, "amadd.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amadd.d", "r,r,r,u0:0", "amadd.d %1,%2,%3", 0, 0, 0},
{0x38618000, 0xffff8000, "amadd.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amand.w", "r,r,r,u0:0", "amand.w %1,%2,%3", 0, 0, 0},
{0x38620000, 0xffff8000, "amand.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amand.d", "r,r,r,u0:0", "amand.d %1,%2,%3", 0, 0, 0},
{0x38628000, 0xffff8000, "amand.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amor.w", "r,r,r,u0:0", "amor.w %1,%2,%3", 0, 0, 0},
{0x38630000, 0xffff8000, "amor.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amor.d", "r,r,r,u0:0", "amor.d %1,%2,%3", 0, 0, 0},
{0x38638000, 0xffff8000, "amor.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amxor.w", "r,r,r,u0:0", "amxor.w %1,%2,%3", 0, 0, 0},
{0x38640000, 0xffff8000, "amxor.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amxor.d", "r,r,r,u0:0", "amxor.d %1,%2,%3", 0, 0, 0},
{0x38648000, 0xffff8000, "amxor.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammax.w", "r,r,r,u0:0", "ammax.w %1,%2,%3", 0, 0, 0},
{0x38650000, 0xffff8000, "ammax.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammax.d", "r,r,r,u0:0", "ammax.d %1,%2,%3", 0, 0, 0},
{0x38658000, 0xffff8000, "ammax.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammin.w", "r,r,r,u0:0", "ammin.w %1,%2,%3", 0, 0, 0},
{0x38660000, 0xffff8000, "ammin.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammin.d", "r,r,r,u0:0", "ammin.d %1,%2,%3", 0, 0, 0},
{0x38668000, 0xffff8000, "ammin.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammax.wu", "r,r,r,u0:0", "ammax.wu %1,%2,%3", 0, 0, 0},
{0x38670000, 0xffff8000, "ammax.wu", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammax.du", "r,r,r,u0:0", "ammax.du %1,%2,%3", 0, 0, 0},
{0x38678000, 0xffff8000, "ammax.du", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammin.wu", "r,r,r,u0:0", "ammin.wu %1,%2,%3", 0, 0, 0},
{0x38680000, 0xffff8000, "ammin.wu", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammin.du", "r,r,r,u0:0", "ammin.du %1,%2,%3", 0, 0, 0},
{0x38688000, 0xffff8000, "ammin.du", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amswap_db.w", "r,r,r,u0:0", "amswap_db.w %1,%2,%3", 0, 0, 0},
{0x38690000, 0xffff8000, "amswap_db.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amswap_db.d", "r,r,r,u0:0", "amswap_db.d %1,%2,%3", 0, 0, 0},
{0x38698000, 0xffff8000, "amswap_db.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amadd_db.w", "r,r,r,u0:0", "amadd_db.w %1,%2,%3", 0, 0, 0},
{0x386a0000, 0xffff8000, "amadd_db.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amadd_db.d", "r,r,r,u0:0", "amadd_db.d %1,%2,%3", 0, 0, 0},
{0x386a8000, 0xffff8000, "amadd_db.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amand_db.w", "r,r,r,u0:0", "amand_db.w %1,%2,%3", 0, 0, 0},
{0x386b0000, 0xffff8000, "amand_db.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amand_db.d", "r,r,r,u0:0", "amand_db.d %1,%2,%3", 0, 0, 0},
{0x386b8000, 0xffff8000, "amand_db.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amor_db.w", "r,r,r,u0:0", "amor_db.w %1,%2,%3", 0, 0, 0},
{0x386c0000, 0xffff8000, "amor_db.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amor_db.d", "r,r,r,u0:0", "amor_db.d %1,%2,%3", 0, 0, 0},
{0x386c8000, 0xffff8000, "amor_db.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amxor_db.w", "r,r,r,u0:0", "amxor_db.w %1,%2,%3", 0, 0, 0},
{0x386d0000, 0xffff8000, "amxor_db.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "amxor_db.d", "r,r,r,u0:0", "amxor_db.d %1,%2,%3", 0, 0, 0},
{0x386d8000, 0xffff8000, "amxor_db.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammax_db.w", "r,r,r,u0:0", "ammax_db.w %1,%2,%3", 0, 0, 0},
{0x386e0000, 0xffff8000, "ammax_db.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammax_db.d", "r,r,r,u0:0", "ammax_db.d %1,%2,%3", 0, 0, 0},
{0x386e8000, 0xffff8000, "ammax_db.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammin_db.w", "r,r,r,u0:0", "ammin_db.w %1,%2,%3", 0, 0, 0},
{0x386f0000, 0xffff8000, "ammin_db.w", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammin_db.d", "r,r,r,u0:0", "ammin_db.d %1,%2,%3", 0, 0, 0},
{0x386f8000, 0xffff8000, "ammin_db.d", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammax_db.wu", "r,r,r,u0:0", "ammax_db.wu %1,%2,%3", 0, 0, 0},
{0x38700000, 0xffff8000, "ammax_db.wu", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammax_db.du", "r,r,r,u0:0", "ammax_db.du %1,%2,%3", 0, 0, 0},
{0x38708000, 0xffff8000, "ammax_db.du", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammin_db.wu", "r,r,r,u0:0", "ammin_db.wu %1,%2,%3", 0, 0, 0},
{0x38710000, 0xffff8000, "ammin_db.wu", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0, 0, "ammin_db.du", "r,r,r,u0:0", "ammin_db.du %1,%2,%3", 0, 0, 0},
{0x38718000, 0xffff8000, "ammin_db.du", "r0:5,r10:5,r5:5", 0, 0, 0, 0},
{0x38720000, 0xffff8000, "dbar", "u0:15", 0, 0, 0, 0},
{0x38728000, 0xffff8000, "ibar", "u0:15", 0, 0, 0, 0},
{0x38740000, 0xffff8000, "fldgt.s", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38748000, 0xffff8000, "fldgt.d", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38750000, 0xffff8000, "fldle.s", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38758000, 0xffff8000, "fldle.d", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38760000, 0xffff8000, "fstgt.s", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38768000, 0xffff8000, "fstgt.d", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38770000, 0xffff8000, "fstle.s", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38778000, 0xffff8000, "fstle.d", "f0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38780000, 0xffff8000, "ldgt.b", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38788000, 0xffff8000, "ldgt.h", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38790000, 0xffff8000, "ldgt.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x38798000, 0xffff8000, "ldgt.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387a0000, 0xffff8000, "ldle.b", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387a8000, 0xffff8000, "ldle.h", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387b0000, 0xffff8000, "ldle.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387b8000, 0xffff8000, "ldle.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387c0000, 0xffff8000, "stgt.b", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387c8000, 0xffff8000, "stgt.h", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387d0000, 0xffff8000, "stgt.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387d8000, 0xffff8000, "stgt.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387e0000, 0xffff8000, "stle.b", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387e8000, 0xffff8000, "stle.h", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387f0000, 0xffff8000, "stle.w", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0x387f8000, 0xffff8000, "stle.d", "r0:5,r5:5,r10:5", 0, 0, 0, 0},
{0} /* Terminate the list.  */
};

static struct loongarch_opcode loongarch_jmp_opcodes[] = {
/* match,    mask,       name, format, macro, include, exclude, pinfo */
{0, 0, "beqz", "r,la", "beqz %1,%%pcrel(%2)", 0, 0, 0},
{0x40000000, 0xfc000000, "beqz", "r5:5,sb0:5|10:16<<2", 0, 0, 0, 0},
{0, 0, "bnez", "r,la", "bnez %1,%%pcrel(%2)", 0, 0, 0},
{0x44000000, 0xfc000000, "bnez", "r5:5,sb0:5|10:16<<2", 0, 0, 0, 0},
{0, 0, "bceqz", "c,la", "bceqz %1,%%pcrel(%2)", 0, 0, 0},
{0x48000000, 0xfc000300, "bceqz", "c5:3,sb0:5|10:16<<2", 0, 0, 0, 0},
{0, 0, "bcnez", "c,la", "bcnez %1,%%pcrel(%2)", 0, 0, 0},
{0x48000100, 0xfc000300, "bcnez", "c5:3,sb0:5|10:16<<2", 0, 0, 0, 0},
{0x48000200, 0xfc0003e0, "jiscr0", "s0:5|10:16<<2", 0, 0, 0, 0},
{0x48000300, 0xfc0003e0, "jiscr1", "s0:5|10:16<<2", 0, 0, 0, 0},
{0, 0, "jr", "r", "jirl $r0,%1,0", 0, 0, 0},
{0x4c000000, 0xfc000000, "jirl", "r0:5,r5:5,s10:16<<2", 0, 0, 0, 0},
{0, 0, "b", "la", "b %%pcrel(%1)", 0, 0, 0},
{0x50000000, 0xfc000000, "b", "sb0:10|10:16<<2", 0, 0, 0, 0},
{0, 0, "bl", "la", "bl %%pcrel(%1)", 0, 0, 0},
{0x54000000, 0xfc000000, "bl", "sb0:10|10:16<<2", 0, 0, 0, 0},
{0, 0, "beq", "r,r,la", "beq %1,%2,%%pcrel(%3)", 0, 0, 0},
{0x58000000, 0xfc000000, "beq", "r5:5,r0:5,sb10:16<<2", 0, 0, 0, 0},
{0, 0, "bne", "r,r,la", "bne %1,%2,%%pcrel(%3)", 0, 0, 0},
{0x5c000000, 0xfc000000, "bne", "r5:5,r0:5,sb10:16<<2", 0, 0, 0, 0},
{0, 0, "blt", "r,r,la", "blt %1,%2,%%pcrel(%3)", 0, 0, 0},
{0x60000000, 0xfc000000, "blt", "r5:5,r0:5,sb10:16<<2", 0, 0, 0, 0},
{0, 0, "bge", "r,r,la", "bge %1,%2,%%pcrel(%3)", 0, 0, 0},
{0x64000000, 0xfc000000, "bge", "r5:5,r0:5,sb10:16<<2", 0, 0, 0, 0},
{0, 0, "bltu", "r,r,la", "bltu %1,%2,%%pcrel(%3)", 0, 0, 0},
{0x68000000, 0xfc000000, "bltu", "r5:5,r0:5,sb10:16<<2", 0, 0, 0, 0},
{0, 0, "bgeu", "r,r,la", "bgeu %1,%2,%%pcrel(%3)", 0, 0, 0},
{0x6c000000, 0xfc000000, "bgeu", "r5:5,r0:5,sb10:16<<2", 0, 0, 0, 0},
{0, 0, "fbceqz", "c,s", "bceqz %1,%2", 0, 0, 0},
{0, 0, "fbceqz", "c,l", "bceqz %1,%2", 0, 0, 0},
{0, 0, "vbceqz", "c,s", "bceqz %1,%2", 0, 0, 0},
{0, 0, "vbceqz", "c,l", "bceqz %1,%2", 0, 0, 0},
{0, 0, "xvbceqz", "c,s", "bceqz %1,%2", 0, 0, 0},
{0, 0, "xvbceqz", "c,l", "bceqz %1,%2", 0, 0, 0},
{0, 0, "fbcnez", "c,s", "bcnez %1,%2", 0, 0, 0},
{0, 0, "fbcnez", "c,l", "bcnez %1,%2", 0, 0, 0},
{0, 0, "vbcnez", "c,s", "bcnez %1,%2", 0, 0, 0},
{0, 0, "vbcnez", "c,l", "bcnez %1,%2", 0, 0, 0},
{0, 0, "xvbcnez", "c,s", "bcnez %1,%2", 0, 0, 0},
{0, 0, "xvbcnez", "c,l", "bcnez %1,%2", 0, 0, 0},
{0} /* Terminate the list.  */
};

struct loongarch_ase loongarch_ASEs[] = {
{&LARCH_opts.ase_test, loongarch_test_opcodes, 0, 0, {0}, 0, 0},
{&LARCH_opts.ase_fix, loongarch_macro_opcodes, 0, 0, {0}, 0, 0},
{&LARCH_opts.ase_fix, loongarch_lmm_opcodes, 0, 0, {0}, 0, 0},
{&LARCH_opts.ase_fix, loongarch_privilege_opcodes, 0, 0, {0}, 0, 0},
{&LARCH_opts.ase_fix, loongarch_jmp_opcodes, 0, 0, {0}, 0, 0},
{&LARCH_opts.ase_fix, loongarch_load_store_opcodes, 0, 0, {0}, 0, 0},
{&LARCH_opts.ase_fix, loongarch_fix_opcodes, 0, 0, {0}, 0, 0},
{&LARCH_opts.ase_float, loongarch_4opt_opcodes, 0, 0, {0}, 0, 0},
{&LARCH_opts.ase_float, loongarch_float_opcodes, 0, 0, {0}, 0, 0},

{0},
};

